clear; close all; 
%% CONSTANTS
scale = 1.5;  % scale of Gabors, length = 17 pixels
norient = 15;  % number of orientations
epsilon = .1;  % allowed correlation between selected Gabors
Totalsketch = 14;  % allowed number of selected Gabors   
Upperbound = 16.; % saturation value
sub = 2;    % subsampling rate for Gabor
Lrange = 3; % allowed shifting in location (times sub pixels)
Orange = 1; % allowed shifting in orientation
%%% Note: Lrange*sub is the pixels allowed to shift along normal
%%%       Lrange*sub should be the same in learning and testing
LIK = 1.; SHUTUP = 0.;  % use log-likelihood score. If LIK=0, then use active correlation
% SHUTUP is to neglect the positions with low average responses          
% LIK = 0; SHUTUP = 1.;   
fold = .6; 
outputFolder = 'deerLarge';
mkdir([outputFolder 'eps']); 
mkdir([outputFolder 'png']); 
%% TRAINING EXAMPLES 
Folder = 'd';  % image input folder
Iname = dir([Folder '\*.jpg']);
nimage = size(Iname, 1);
Isize = zeros(nimage, 2);
for (i = 1 : nimage)
    I{i} = imresize(double(rgb2gray(imread([Folder '\' Iname(i).name]))), fold, 'nearest');
    %Ishow{i} = imread([Folder '\' Iname(i).name]); 
    Isize(i, :) = size(I{i});
end
sx = min(Isize(:, 1)); sy = min(Isize(:, 2)); 
for (i = 1 : nimage)
    I{i} = I{i}(1:sx, 1:sy); 
end   % make the sizes of the training images the same
%% FILTERING IMAGES 
disp(['start filtering']);
tic
[allfilter, allsymbol] = makefilter(scale, norient);  % generate Gabor filters 
h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor
C = corr(allfilter, epsilon);    % generate the inhibition maps
allfiltered = applyfilterfftsame(I, allfilter);  % filter training images
disp(['filtering time: ' num2str(toc) ' seconds']);
%% VARIABLES FOR MEX-C 
Mi = zeros(1, Totalsketch);  % orientation of selected Gabor
Mx = zeros(1, Totalsketch);  % position of selected Gabor
My = zeros(1, Totalsketch); 
Mm = zeros(1, Totalsketch);  % average response of selected Gabor
sym = zeros(size(I{1}));  % symbolic plot of selected Gabor
Asym = cell(1, nimage);   % symbolic plot for each image with active Gabor
for (i = 1 : nimage)
    Asym{i} = zeros(size(I{1}));  
end
tss = zeros(nimage, 1);  % total scores for testing images
gain = zeros(Totalsketch, 1);  % coding gain for selected Gabor

Ssym = cell(1, Totalsketch); 
for (i = 1 : Totalsketch)
    Ssym{i} = zeros(size(I{1}));  
end

binnum = 50; % number of bins 
%% MEX-C 
mex Clearn.c;   % compile C code
disp(['start learning']);
tic
Clearn(nimage, norient, allfiltered, C, h, sx, sy, Mi, Mx, My, Mm, allsymbol(1, :), sym, Asym, Lrange, Orange, sub, Totalsketch, Ssym, Upperbound, tss, gain, LIK, SHUTUP, binnum);
disp(['mex-C learning time: ' num2str(toc) ' seconds']);
%% RESULTS 
save 'model' norient h sx sy Mi Mx My Mm allfilter allsymbol Lrange Orange sub Totalsketch Upperbound LIK; 
%% save figures into eps and png folders  
numImage = nimage; imageFolder = Folder; imageName = dir([imageFolder '/*.jpg']);
pic = -double(sym); showImage; 
saveas(gcf, [outputFolder 'eps/template.eps'], 'eps');
saveas(gcf, [outputFolder 'png/template.png'], 'png');
pic = sym*0.+255; showImage; 
saveas(gcf, [outputFolder 'eps/empty.eps'], 'eps');
saveas(gcf, [outputFolder 'png/empty.png'], 'png');
for (img = 1 : numImage)
    tmpIm = imresize(imread([imageFolder '/' imageName(img).name]), fold, 'nearest');
    pic = tmpIm(1:sx, 1:sy, :); showImage; 
    saveas(gcf, [outputFolder 'eps/input' num2str(100+img) '.eps'], 'epsc');
    saveas(gcf, [outputFolder 'png/input' num2str(100+img) '.png'], 'png');    
    pic = -(Asym{img}); showImage; 
    saveas(gcf, [outputFolder 'eps/sketch' num2str(100+img) '.eps'], 'eps');
    saveas(gcf, [outputFolder 'png/sketch' num2str(100+img) '.png'], 'png');
end
for (i = 1:Totalsketch)            
    pic =  -Ssym{i}; showImage; 
    saveas(gcf, [outputFolder 'eps/template' num2str(100+i) '.eps'], 'eps');
    saveas(gcf, [outputFolder 'png/template' num2str(100+i) '.png'], 'png'); 
end
%% write the html code for reproducibility page
heightstr = '"height=100> '; 
disp(['<IMG SRC="' outputFolder 'png/template' '.png' heightstr]);
disp(['<IMG SRC="' outputFolder 'png/empty' '.png' heightstr]);
counter = 1; length = 5; 
for (img = 1 : numImage)
 disp(['<IMG SRC="' outputFolder 'png/input' num2str(100+img) '.png' heightstr]);
 disp(['<IMG SRC="' outputFolder 'png/sketch' num2str(100+img) '.png' heightstr]);
 counter = counter + 1; 
 if (counter == length)
     disp('<BR>'); 
     counter = 0; 
 end
end
for (i = 1:Totalsketch)            
 disp(['<IMG SRC="' outputFolder 'png/template' num2str(100+i) '.png' heightstr]);
end
%% write the LaTex code for paper 
heightstr = 'height=.6in}'; 
disp(['\psfig{figure=' outputFolder 'eps/template' '.eps,'  heightstr]); 
disp(['\psfig{figure=' outputFolder 'eps/empty' '.eps,'  heightstr]);
counter = 1; length = 5; 
for (img = 1 : numImage)
 disp(['\psfig{figure=' outputFolder 'eps/input' num2str(100+img) '.eps,' heightstr]);
 disp(['\psfig{figure=' outputFolder 'eps/sketch' num2str(100+img) '.eps,' heightstr]);
 counter = counter + 1; 
 if (counter == length)
     disp('\\'); 
     counter = 0; 
 end
end
for (i = 1:Totalsketch)            
  disp(['\psfig{figure=' outputFolder 'eps/template' num2str(100+i) '.eps,' heightstr]);
end
